# Code for running MatrixEQTL 
# please email sellis18 at jhmi dot edu with any questions/bugs/suggestions

#to run:
#Rscript eQTL_Gold_Standards_BMCGenomics.r > GW_Brain_Step1.log

library(MatrixEQTL)
library(qvalue)


##################################################
########## PREVIOUSLY REPORTED EQTLS #############
##################################################
##### list(s) of previously identified eQTLs #####
## Brain ##
# Brain eQTLs obtained from the following two publications: 
#  http://www.plosgenetics.org/article/info%3Adoi%2F10.1371%2Fjournal.pgen.1002707 [Zou, et. al, PLOS Genetics June, 2012]
#  http://www.ncbi.nlm.nih.gov/pmc/articles/PMC3365261/ [Kim, et. al, Translational Psychiatry, May 2012]
# Supplementary Table 3

## LCL ##
# known eQTLs to compare to Blood GTeX data from the following publication:
#  http://bioinformatics.oxfordjournals.org/content/28/3/451 [Xia, et. al, Bioinformatics, Feb 2012]
# Supplementary Table 4

#read in previously-identified eQTL information
knowneQTLs_Brain <- read.table("supp3.txt",header=TRUE, row.names=NULL)
#knowneQTLs_LCL <- read.table("supp4.txt", sep="\t",header=TRUE, row.names=NULL)
# or read in file with eQTLs from your tissue of interest
#lists formatted such that rsID in column 1 and Ensembl gene name in column 2


##################################################
######### RNA-SEQ EXPRESSION DATA ################
##################################################
#here, I've included our mapped RNA-Seq data (log2 gene counts) 
	#pergeneNAs removed
	#Normalized by CQN
	#for the genes tested in the Brain data
	## includes genes in knowneQTLs_Brain list above
# FORMAT: genes in rows; samples in columns
# expression_Brain.txt


##################################################
############### GENOTYPING DATA ##################
##################################################
### data are from Affymetrix Human 6.0 Array ###
## includes SNPs in knowneQTLs_Brain list above

# FORMAT: snps in rows; samples in columns
# SNPs_Brain.txt

###################################################
################## STEP FOUR ######################
###################################################
# prepare position and covariates files for MatrixEQTL
# Note: coordinates are hg19 in files provided

# SNP POSISITONS IN : snppos.txt 
#File Format:
#Column 1: SNP
#Column 2: chromosome (ie "chr1")
#Column 3: SNP position

# GENE POSITIONS IN : genepos.txt
# File Format: 
# Column 1: Ensembl Gene NAme
# Column 2: chromosome (ie "chr1")
# Column 3: Gene Start position
# Column 4: Gene Stop position

# COVARIATES IN: covariates.txt
# file includes, age, sex, site, 10PCs
# covariates in rows;samples in columns

##################################################
################## STEP FIVE #####################
##################################################

##### Running MatrixEQTL #####
# Linear model to use, modelANOVA, modelLINEAR, or modelLINEAR_CROSS
useModel = modelLINEAR ; # modelANOVA, modelLINEAR, or modelLINEAR_CROSS

# Genotype file name
SNP_file_name = "SNPs_Brain.txt";
snps_location_file_name = "snppos.txt";
snpspos = read.table(snps_location_file_name, header = TRUE, stringsAsFactors = FALSE);

# Gene expression file name
expression_file_name = "expression_Brain.txt";					
gene_location_file_name = "genepos.txt";
genepos = read.table(gene_location_file_name, header = TRUE, stringsAsFactors = FALSE);

# Covariates file name
# Set to character() for no covariates
covariates_file_name = "covariates.txt";
#covariates_file_name=character();

# Error covariance matrix
# Set to numeric() for identity.
errorCovariance = numeric();
# errorCovariance = read.table("Sample_Data/errorCovariance.txt");

cisDist = 1e6;								

## Load genotype data

snps = SlicedData$new();
snps$fileDelimiter = "\t"; # the TAB character
snps$fileOmitCharacters = "NA"; # denote missing values;
snps$fileSkipRows = 1; # one row of column labels
snps$fileSkipColumns = 1; # one column of row labels
snps$fileSliceSize = 2000; # read file in pieces of 2,000 rows
snps$LoadFile(SNP_file_name);

## Load gene expression data
gene = SlicedData$new();
gene$fileDelimiter = "\t"; # the TAB character
gene$fileOmitCharacters = "NA"; # denote missing values;
gene$fileSkipRows = 1; # one row of column labels
gene$fileSkipColumns = 1; # one column of row labels
gene$fileSliceSize = 2000; # read file in pieces of 2,000 rows
gene$LoadFile(expression_file_name);	

## Load covariates

cvrt = SlicedData$new();
cvrt$fileDelimiter = "\t"; # the TAB character
cvrt$fileOmitCharacters = "NA"; # denote missing values;
cvrt$fileSkipRows = 1; # one row of column labels
cvrt$fileSkipColumns = 1; # one column of row labels
cvrt$fileSliceSize = 2000; # read file in one piece
if(length(covariates_file_name)>0) {
    cvrt$LoadFile(covariates_file_name);
}

# Only ass_age.sex.PCs1-8ociations significant at this level will be saved
pvOutputThreshold = 0;
pvOutputThreshold.cis = 1;				# set to 1 to get ALL associations
pvOutputThreshold.tra = 0;				# set to 0 to do only cis-eQTL analysis

## Run the analysis
### Cis-eQTL only ###

# Output file name
output_file_name = "eQTL_output.txt";
output_file_name.cis  = "eQTL_output_CIS.txt";

me = Matrix_eQTL_main(
	snps,
	gene,
	cvrt,
	output_file_name = output_file_name,
	pvOutputThreshold = pvOutputThreshold,
	useModel = modelLINEAR,
	errorCovariance = numeric(),
	verbose = TRUE,
	output_file_name.cis = output_file_name.cis,
	pvOutputThreshold.cis = pvOutputThreshold.cis,
	snpspos ,
	genepos ,
	cisDist = cisDist,
	pvalue.hist = "qqplot")

# The output file from this analysis (ie "eQTL_output_CIS.txt") will be the input for the lambda correction script


##########################################################################
#### Determining # of eQTLs that replicate from previous publications ####
########## & to determine genome-wide inflation factor ###################
##########################################################################
# run eQTL_genomewide_Gold_Standards_BMCGenomics.r 
	# change input file names there as needed for your data

q()
n
